@@ -40,29 +40,26 @@ fi
 # without trying to tell you about what it's doing. So unless it hangs
 # for much longer than that, don't worry, it's likely to be normal.
 if [ "$1" != --load ]; then
+    if [ "x$1" != x ]; then
+        echo Unknown option \'"$1"\' to make-target-2
+        exit 1
+    fi
     echo //doing warm init - compilation phase
-    echo '(load "loader.lisp") (load-sbcl-file "src/cold/warm.lisp")' | \
-    ./src/runtime/sbcl \
-        --core output/cold-sbcl.core \
-        --lose-on-corruption \
-        --no-sysinit --no-userinit
+    ./src/runtime/sbcl --core output/cold-sbcl.core \
+     --lose-on-corruption --no-sysinit --no-userinit \
+     --eval '(sb-fasl::!warm-load "src/cold/warm.lisp")' --quit
 fi
 echo //doing warm init - load and dump phase
-echo '(load "loader.lisp") (load-sbcl-file "make-target-2-load.lisp" nil)
-#+gencgc(setf (extern-alien "gc_coalesce_string_literals" char) 2)
-(sb-ext:save-lisp-and-die "output/sbcl.core")' | \
-./src/runtime/sbcl \
---core output/cold-sbcl.core \
---lose-on-corruption \
---no-sysinit --no-userinit
+./src/runtime/sbcl --core output/cold-sbcl.core \
+ --lose-on-corruption --no-sysinit --no-userinit \
+ --eval '(sb-fasl::!warm-load "make-target-2-load.lisp")' \
+ --eval '(progn #+gencgc(setf (extern-alien "gc_coalesce_string_literals" char) 2))' \
+ --eval '(sb-ext:save-lisp-and-die "output/sbcl.core")'
 
 echo //checking for leftover cold-init symbols
-./src/runtime/sbcl \
---core output/sbcl.core \
---lose-on-corruption \
---noinform \
---no-sysinit --no-userinit \
---eval '(restart-case
+./src/runtime/sbcl --core output/sbcl.core \
+ --lose-on-corruption --noinform --no-sysinit --no-userinit --eval '
+    (restart-case
       (let (l)
         (sb-vm::map-allocated-objects
          (lambda (obj type size)
@@ -71,8 +68,17 @@ echo //checking for leftover cold-init symbols
                       (search "!" (string obj)))
              (push obj l)))
          :all)
-        (format t "Found ~D:~%~S~%" (length l) l))
+        (format t "Found ~D:~%~S~%" (length l) l)
+        (sb-int:awhen
+          (mapcan (quote apropos-list)
+           (quote ("DEFINE-INFO-TYPE" "LVAR-TYPE-USING"
+                   "TWO-ARG-+/-"
+                   "PPRINT-TAGBODY-GUTS" "WITH-DESCRIPTOR-HANDLERS"
+                   "SUBTRACT-BIGNUM-LOOP" "BIGNUM-REPLACE" "WITH-BIGNUM-BUFFERS"
+                   "GCD-ASSERT" "MODULARLY" "BIGNUM-NEGATE-LOOP"
+                   "SHIFT-RIGHT-UNALIGNED"
+                   "STRING-LESS-GREATER-EQUAL-TESTS")))
+         (format t "~&Leftover from [disabled?] tree-shaker:~%~S~%" sb-int:it)))
     (abort ()
       :report "Abort building SBCL."
-      (sb-ext:exit :code 1)))' \
---quit
+      (sb-ext:exit :code 1)))' --quit
