@@ -13,6 +13,20 @@
 
 ;;;; DYNAMIC-USAGE and friends
 
+#!+(and relocatable-heap gencgc)
+(define-alien-variable ("DYNAMIC_SPACE_START" sb!vm:dynamic-space-start) unsigned-long)
+#!-sb-fluid
+(declaim (inline current-dynamic-space-start))
+(defun current-dynamic-space-start ()
+  #!+gencgc sb!vm:dynamic-space-start
+  #!-gencgc (extern-alien "current_dynamic_space" unsigned-long))
+
+#!+(or x86 x86-64)
+(progn
+  (declaim (inline dynamic-space-free-pointer))
+  (defun dynamic-space-free-pointer ()
+    (extern-alien "dynamic_space_free_pointer" system-area-pointer)))
+
 #!-sb-fluid
 (declaim (inline dynamic-usage))
 #!+gencgc
@@ -25,72 +39,24 @@
                 (current-dynamic-space-start))))
 
 (defun static-space-usage ()
-  (- (ash sb!vm:*static-space-free-pointer* sb!vm:n-fixnum-tag-bits)
-     sb!vm:static-space-start))
+  (- (sap-int sb!vm:*static-space-free-pointer*) sb!vm:static-space-start))
 
 (defun read-only-space-usage ()
-  (- (ash sb!vm::*read-only-space-free-pointer* sb!vm:n-fixnum-tag-bits)
-     sb!vm:read-only-space-start))
+  (- (sap-int sb!vm:*read-only-space-free-pointer*) sb!vm:read-only-space-start))
+
+;;; Convert the descriptor into a SAP. The bits all stay the same, we just
+;;; change our notion of what we think they are.
+(declaim (inline descriptor-sap))
+(defun descriptor-sap (x) (int-sap (get-lisp-obj-address x)))
 
 (defun control-stack-usage ()
   #!-stack-grows-downward-not-upward
-  (- (sap-int (sb!c::control-stack-pointer-sap))
-     (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-start*)))
+  (sap- (control-stack-pointer-sap) (descriptor-sap sb!vm:*control-stack-start*))
   #!+stack-grows-downward-not-upward
-  (- (sap-int (sb!di::descriptor-sap sb!vm:*control-stack-end*))
-     (sap-int (sb!c::control-stack-pointer-sap))))
+  (sap- (descriptor-sap sb!vm:*control-stack-end*) (control-stack-pointer-sap)))
 
 (defun binding-stack-usage ()
-  (- (sap-int (sb!c::binding-stack-pointer-sap))
-     (sap-int (sb!di::descriptor-sap sb!vm:*binding-stack-start*))))
-
-;;;; ROOM
-
-(defun room-minimal-info ()
-  (format t "Dynamic space usage is:   ~10:D bytes.~%" (dynamic-usage))
-  (format t "Read-only space usage is: ~10:D bytes.~%" (read-only-space-usage))
-  (format t "Static space usage is:    ~10:D bytes.~%" (static-space-usage))
-  (format t "Control stack usage is:   ~10:D bytes.~%" (control-stack-usage))
-  (format t "Binding stack usage is:   ~10:D bytes.~%" (binding-stack-usage))
-  #!+sb-thread
-  (format t
-          "Control and binding stack usage is for the current thread only.~%")
-  (format t "Garbage collection is currently ~:[enabled~;DISABLED~].~%"
-          *gc-inhibit*))
-
-(defun room-intermediate-info ()
-  (room-minimal-info)
-  (sb!vm:memory-usage :count-spaces '(:dynamic)
-                      :print-spaces t
-                      :cutoff 0.05f0
-                      :print-summary nil))
-
-(defun room-maximal-info ()
-  ;; FIXME: SB!VM:INSTANCE-USAGE calls suppressed until bug 344 is fixed
-  (room-intermediate-info)
-  ;; old way, could be restored when bug 344 fixed:
-  ;;x (room-minimal-info)
-  ;;x (sb!vm:memory-usage :count-spaces '(:static :dynamic))
-  ;;x (sb!vm:instance-usage :dynamic :top-n 10)
-  ;;x (sb!vm:instance-usage :static :top-n 10)
-  )
-
-(defun room (&optional (verbosity :default))
-  "Print to *STANDARD-OUTPUT* information about the state of internal
-  storage and its management. The optional argument controls the
-  verbosity of output. If it is T, ROOM prints out a maximal amount of
-  information. If it is NIL, ROOM prints out a minimal amount of
-  information. If it is :DEFAULT or it is not supplied, ROOM prints out
-  an intermediate amount of information."
-  (fresh-line)
-  (ecase verbosity
-    ((t)
-     (room-maximal-info))
-    ((nil)
-     (room-minimal-info))
-    (:default
-     (room-intermediate-info)))
-  (values))
+  (sap- (binding-stack-pointer-sap) (descriptor-sap sb!vm:*binding-stack-start*)))
 
 ;;;; GET-BYTES-CONSED
 
@@ -101,7 +67,7 @@
 ;;; ever allocated by adding this to the number of bytes currently
 ;;; allocated and never freed.)
 (declaim (type unsigned-byte *n-bytes-freed-or-purified*))
-(defvar *n-bytes-freed-or-purified* 0)
+(define-load-time-global *n-bytes-freed-or-purified* 0)
 (defun gc-reinit ()
   (setq *gc-inhibit* nil)
   (gc)
@@ -121,7 +87,7 @@ and submit it as a patch."
 
 ;;;; GC hooks
 
-(!defvar *after-gc-hooks* nil
+(!define-load-time-global *after-gc-hooks* nil
   "Called after each garbage collection, except for garbage collections
 triggered during thread exits. In a multithreaded environment these hooks may
 run in any thread.")
@@ -132,11 +98,11 @@ run in any thread.")
 (define-alien-routine collect-garbage int
   (#!+gencgc last-gen #!-gencgc ignore int))
 
-#!+sb-thread
+#!+(or sb-thread sb-safepoint)
 (progn
   (define-alien-routine gc-stop-the-world void)
   (define-alien-routine gc-start-the-world void))
-#!-sb-thread
+#!-(or sb-thread sb-safepoint)
 (progn
   (defun gc-stop-the-world ())
   (defun gc-start-the-world ()))
@@ -167,6 +133,9 @@ statistics are appended to it."
 (defun dynamic-space-size ()
   "Size of the dynamic space in bytes."
   (extern-alien "dynamic_space_size" os-vm-size-t))
+#!+gencgc
+(define-symbol-macro sb!vm:dynamic-space-end
+  (+ (dynamic-space-size) sb!vm:dynamic-space-start))
 
 ;;;; SUB-GC
 
@@ -185,9 +154,9 @@ statistics are appended to it."
 
 ;;; For GENCGC all generations < GEN will be GC'ed.
 
-(defvar *already-in-gc* (sb!thread:make-mutex :name "GC lock"))
+(define-load-time-global *already-in-gc* (sb!thread:make-mutex :name "GC lock"))
 
-(defun sub-gc (&key (gen 0))
+(defun sub-gc (gen)
   (cond (*gc-inhibit*
          (setf *gc-pending* t)
          nil)
@@ -200,7 +169,7 @@ statistics are appended to it."
                         (new-usage 0)
                         (start-time (get-internal-run-time)))
                     (collect-garbage gen)
-                    (setf *gc-epoch* (cons nil nil))
+                    (setf *gc-epoch* (cons 0 0))
                     (let ((run-time (- (get-internal-run-time) start-time)))
                       ;; KLUDGE: Sometimes we see the second getrusage() call
                       ;; return a smaller value than the first, which can
@@ -208,12 +177,12 @@ statistics are appended to it."
                       ;; turn is a type-error.
                       (when (plusp run-time)
                         (incf *gc-run-time* run-time)))
-                    #!+sb-safepoint
+                    #!+(and sb-thread sb-safepoint)
                     (setf *stop-for-gc-pending* nil)
                     (setf *gc-pending* nil
                           new-usage (dynamic-usage))
                     #!+sb-thread
-                    (assert (not *stop-for-gc-pending*))
+                    (aver (not *stop-for-gc-pending*))
                     (gc-start-the-world)
                     ;; In a multithreaded environment the other threads
                     ;; will see *n-b-f-o-p* change a little late, but
@@ -234,7 +203,6 @@ statistics are appended to it."
              (sb!thread::without-thread-waiting-for
                  (:already-without-interrupts t)
                (let ((sb!impl::*deadline* nil)
-                     (sb!impl::*deadline-seconds* nil)
                      (epoch *gc-epoch*))
                  (loop
                   ;; GCing must be done without-gcing to avoid
@@ -284,7 +252,7 @@ statistics are appended to it."
   ;; another GC. FIXME: it can potentially exceed maximum interrupt
   ;; nesting by triggering GCs.
   ;;
-  ;; Can that be avoided by having the finalizers and hooks run only
+  ;; Can that be avoided by having the hooks run only
   ;; from the outermost SUB-GC? If the nested GCs happen in interrupt
   ;; handlers that's not enough.
   ;;
@@ -297,13 +265,26 @@ statistics are appended to it."
   ;;    don't want to run user code in such a case.
   ;;
   ;; The long-term solution will be to keep a separate thread for
-  ;; finalizers and after-gc hooks.
+  ;; after-gc hooks.
+  ;; Finalizers are in a separate thread (usually),
+  ;; but it's not permissible to invoke CONDITION-NOTIFY from a
+  ;; dying thread, so we still need the guard for that, but not
+  ;; the guard for whether interupts are enabled.
   (when (sb!thread:thread-alive-p sb!thread:*current-thread*)
-    (when *allow-with-interrupts*
-      (sb!thread::without-thread-waiting-for ()
-        (with-interrupts
-          (run-pending-finalizers)
-          (call-hooks "after-GC" *after-gc-hooks* :on-error :warn))))))
+    (let ((threadp #!+sb-thread (%instancep sb!impl::*finalizer-thread*)))
+      (when threadp
+        ;; It's OK to frob a condition variable regardless of
+        ;; *allow-with-interrupts*, and probably OK to start a thread.
+        ;; For consistency with the previous behavior, we delay finalization
+        ;; if there is no finalizer thread and interrupts are disabled.
+        ;; That's my excuse anyway, not having looked more in-depth.
+        (run-pending-finalizers))
+      (when *allow-with-interrupts*
+        (sb!thread::without-thread-waiting-for ()
+         (with-interrupts
+           (unless threadp
+             (run-pending-finalizers))
+           (call-hooks "after-GC" *after-gc-hooks* :on-error :warn)))))))
 
 ;;; This is the user-advertised garbage collection function.
 (defun gc (&key (full nil) (gen 0) &allow-other-keys)
@@ -330,7 +311,7 @@ If GEN is provided, it can be used to specify the oldest generation
 guaranteed to be collected."
   #!-gencgc (declare (ignore full))
   (let (#!+gencgc (gen (if full sb!vm:+pseudo-static-generation+ gen)))
-    (when (eq t (sub-gc :gen gen))
+    (when (eq t (sub-gc gen))
       (post-gc))))
 
 (define-alien-routine scrub-control-stack void)
@@ -396,10 +377,6 @@ Note: currently changes to this value are lost when saving core."
 #!+gencgc
 (define-alien-type generation
     (struct generation
-            (alloc-large-start-page page-index-t)
-            (alloc-start-page page-index-t)
-            (alloc-unboxed-start-page page-index-t)
-            #!+segregated-code (alloc-code-start-page page-index-t)
             (bytes-allocated os-vm-size-t)
             (gc-trigger os-vm-size-t)
             (bytes-consed-between-gcs os-vm-size-t)
@@ -486,3 +463,8 @@ Experimental: interface subject to change."
     (alien-funcall (extern-alien "generation_average_age"
                                  (function double generation-index-t))
                    generation))
+
+(declaim (inline sb!vm:is-lisp-pointer))
+(defun sb!vm:is-lisp-pointer (addr) ; Same as is_lisp_pointer() in C
+  #!-64-bit (oddp addr)
+  #!+64-bit (not (logtest (logxor addr 3) 3)))
