@@ -14,12 +14,12 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
   ;; Imports from this package into SB-VM
-  (import '(*condition-name-vec* conditional-opcode
+  (import '(conditional-opcode
             add-sub-immediate-p fixnum-add-sub-immediate-p
             negative-add-sub-immediate-p
             encode-logical-immediate fixnum-encode-logical-immediate
             ldr-str-offset-encodable ldp-stp-offset-p
-            bic-mask extend lsl lsr asr ror @) 'sb!vm)
+            bic-mask extend lsl lsr asr ror @) "SB!VM")
   ;; Imports from SB-VM into this package
   (import '(sb!vm::*register-names*
             sb!vm::add-sub-immediate
@@ -27,10 +27,9 @@
             sb!vm::complex-single-reg sb!vm::complex-double-reg
             sb!vm::tmp-tn sb!vm::zr-tn sb!vm::nsp-offset)))
 
-(setf *disassem-inst-alignment-bytes* 4)
 
 
-(defparameter *conditions*
+(defconstant-eqx +conditions+
   '((:eq . 0)
     (:ne . 1)
     (:cs . 2) (:hs . 2)
@@ -45,25 +44,21 @@
     (:lt . 11)
     (:gt . 12)
     (:le . 13)
-    (:al . 14)))
+    (:al . 14))
+  #'equal)
 
-(defparameter *condition-name-vec*
-  (let ((vec (make-array 16 :initial-element nil)))
-    (dolist (cond *conditions*)
-      (when (null (aref vec (cdr cond)))
-        (setf (aref vec (cdr cond)) (car cond))))
-    vec))
-
-;;; Set assembler parameters. (In CMU CL, this was done with
-;;; a call to a macro DEF-ASSEMBLER-PARAMS.)
-(eval-when (:compile-toplevel :load-toplevel :execute)
-  (setf *assem-scheduler-p* nil))
+(defconstant-eqx sb!vm::+condition-name-vec+
+  #.(let ((vec (make-array 16 :initial-element nil)))
+      (dolist (cond +conditions+ vec)
+        (when (null (aref vec (cdr cond)))
+          (setf (aref vec (cdr cond)) (car cond)))))
+  #'equalp)
 
 (defun conditional-opcode (condition)
-  (cdr (assoc condition *conditions* :test #'eq)))
+  (cdr (assoc condition +conditions+ :test #'eq)))
 
 (defun invert-condition (condition)
-  (aref *condition-name-vec*
+  (aref sb!vm::+condition-name-vec+
         (logxor 1 (conditional-opcode condition))))
 
 ;;;; disassembler field definitions
@@ -250,7 +245,10 @@
   (operand 0 :type (integer 0 63)))
 
 (define-condition cannot-encode-immediate-operand (error)
-  ((value :initarg :value)))
+  ((value :initarg :value))
+  (:report
+   (lambda (condition stream)
+     (format stream "Cannot encode ~S" (slot-value condition 'value)))))
 
 (defun encode-shifted-register (operand)
   (etypecase operand
@@ -431,7 +429,7 @@
 
 (define-instruction-format
     (add-sub-ext-reg 32
-     :default-printer '(:name :tab rd ", " rn ", " extend)
+     :default-printer '(:name :tab rd ", " rn ", " rm extend)
      :include add-sub)
   (op2 :field (byte 8 21) :value #b01011001)
   (extend :fields (list (byte 3 13) (byte 3 10)) :type 'extend)
@@ -541,7 +539,7 @@
   (:printer add-sub-imm ((op #b11) (rd #b11111))
             '('cmp :tab rn ", " imm shift))
   (:printer add-sub-ext-reg ((op #b11) (rd #b11111))
-            '('cmp :tab rn ", " extend))
+            '('cmp :tab rn ", " rm extend))
   (:printer add-sub-shift-reg ((op #b11) (rd #b11111))
             '('cmp :tab rn ", " rm shift))
   (:printer add-sub-shift-reg ((op #b11) (rn #b11111))
@@ -1485,7 +1483,7 @@
                       (fp-reg-type rt1)))))
     (when fp
       (setf size (+ opc 2)))
-    (assert (not (ldb-test (byte size 0) offset)))
+    (aver (not (ldb-test (byte size 0) offset)))
     (emit-ldr-str-pair segment opc v
                        (ecase mode
                          (:post-index #b01)
@@ -1609,13 +1607,13 @@
          (t
           (emit-back-patch segment 4
                            (cond (label
-                                  (assert (label-p label))
+                                  (aver (label-p label))
                                   (lambda (segment posn)
                                     (emit-cond-branch segment
                                                       (ash (- (label-position label) posn) -2)
                                                       (conditional-opcode cond-or-label))))
                                  (t
-                                  (assert (label-p cond-or-label))
+                                  (aver (label-p cond-or-label))
                                   (lambda (segment posn)
                                     (emit-uncond-branch segment
                                                         0
@@ -1687,7 +1685,7 @@
 (define-instruction cbz (segment rt label)
   (:printer compare-branch-imm ((op 0)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (emit-back-patch segment 4
                     (lambda (segment posn)
                       (emit-compare-branch-imm segment
@@ -1699,7 +1697,7 @@
 (define-instruction cbnz (segment rt label)
   (:printer compare-branch-imm ((op 1)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (emit-back-patch segment 4
                     (lambda (segment posn)
                       (emit-compare-branch-imm segment
@@ -1727,7 +1725,7 @@
 (define-instruction tbz (segment rt bit label)
   (:printer test-branch-imm ((op 0)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (check-type bit (integer 0 63))
    (emit-back-patch segment 4
                     (lambda (segment posn)
@@ -1741,7 +1739,7 @@
 (define-instruction tbnz (segment rt bit label)
   (:printer test-branch-imm ((op 1)))
   (:emitter
-   (assert (label-p label))
+   (aver (label-p label))
    (check-type bit (integer 0 63))
    (emit-back-patch segment 4
                     (lambda (segment posn)
@@ -1794,8 +1792,8 @@
   (rd :field (byte 5 0) :type 'x-reg))
 
 (defun emit-pc-relative-inst (op segment rd label &optional (offset 0))
-  (assert (label-p label))
-  (assert (register-p rd))
+  (aver (label-p label))
+  (aver (register-p rd))
   (emit-back-patch segment 4
                    (lambda (segment posn)
                      (let ((offset (+ (- (label-position label) posn)
@@ -1961,7 +1959,7 @@
   `(define-instruction ,name (segment rn rm)
      (:printer fp-compare ((op ,op)))
      (:printer fp-compare ((op ,op) (z 1) (type 0))
-               '(:name :tab rn ", " 0s0))
+               '(:name :tab rn ", " 0f0))
      (:printer fp-compare ((op ,op) (z 1) (type 1))
                '(:name :tab rn ", " 0d0))
      (:emitter
@@ -2272,7 +2270,8 @@
                                  (ldb (byte 2 0) delta)
                                  (ldb (byte 19 2) delta)
                                  (tn-offset dest))))
-           (multi-instruction-maybe-shrink (segment posn magic-value)
+           (multi-instruction-maybe-shrink (segment chooser posn magic-value)
+             (declare (ignore chooser))
              (when (typep (funcall compute-delta posn magic-value)
                           '(signed-byte 21))
                (emit-back-patch segment 4
@@ -2285,6 +2284,7 @@
 
 (define-instruction compute-code (segment code lip object-label)
   (:vop-var vop)
+  (:declare (ignore object-label))
   (:emitter
    (emit-compute segment vop code lip
                  (lambda (position &optional magic-value)
@@ -2334,7 +2334,8 @@
                                 (ldb (byte 19 0)
                                      (ash (compute-delta position) -2))
                                 (tn-offset dest)))
-            (multi-instruction-maybe-shrink (segment posn magic-value)
+            (multi-instruction-maybe-shrink (segment chooser posn magic-value)
+              (declare (ignore chooser))
               (let ((delta (compute-delta posn magic-value)))
                 (when (typep delta '(signed-byte 21))
                   (emit-back-patch segment 4
@@ -2557,16 +2558,20 @@
   (stable-sort constants #'> :key (lambda (constant)
                                     (size-nbyte (caar constant)))))
 
-(defun emit-inline-constant (constant label)
+(defun emit-inline-constant (section constant label)
   (let* ((type (car constant))
          (size (size-nbyte type)))
-    (emit-alignment (integer-length (1- size)))
-    (emit-label label)
-    (let ((val (cdr constant)))
-      (case type
-        (:fixup
-         (inst word (apply #'make-fixup val)))
-        (t
-         (loop repeat size
-               do (inst byte (ldb (byte 8 0) val))
-                  (setf val (ash val -8))))))))
+    (emit section
+          `(.align ,(integer-length (1- size)))
+          label
+          (let ((val (cdr constant)))
+            (case type
+              (:fixup
+               ;; Use the WORD emitter which knows how to emit fixups
+               `(|word| ,(apply #'make-fixup val)))
+              (t
+               ;; Could add pseudo-ops for .WORD, .INT, .QUAD, .OCTA just like gcc has.
+               ;; But it works fine to emit as a sequence of bytes
+               `(.byte ,@(loop repeat size
+                               collect (prog1 (ldb (byte 8 0) val)
+                                         (setf val (ash val -8)))))))))))
