@@ -10,10 +10,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-;;; Given the presence of docstrings and source locations,
-;;; this logic arguably belongs to the runtime kernel, not the compiler,
-;;; but such nuance isn't hugely important.
-(in-package "SB!C")
+(in-package "SB!IMPL")
 
 ;;; Similar to FUNCTION, but the result type is "exactly" specified:
 ;;; if it is an object type, then the function returns exactly one
@@ -49,8 +46,9 @@
 ;;; A map from info-number to its META-INFO object.
 ;;; The reverse mapping is obtained by reading the META-INFO.
 (declaim (type (simple-vector #.(ash 1 info-number-bits)) *info-types*))
-(!defglobal *info-types*
-            (make-array (ash 1 info-number-bits) :initial-element nil))
+(!define-load-time-global *info-types*
+            ;; Must be dumped as a literal for cold-load.
+            #.(make-array (ash 1 info-number-bits) :initial-element nil))
 
 (defstruct (meta-info
             (:constructor
@@ -85,24 +83,34 @@
 ;; Refer to info-vector.lisp for the meaning of this constant.
 (defconstant +no-auxilliary-key+ 0)
 
-;; Return the globaldb info for SYMBOL. With respect to the state diagram
-;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
-;; info slot is LISTP, it is in state 1 or 3. Either way, take the CDR.
-;; Otherwise, it is in state 2 so return the value as-is.
-;; In terms of this function being named "-vector", implying always a vector,
-;; it is understood that NIL is a proxy for +NIL-PACKED-INFOS+, a vector.
-;;
+;;; Return the globaldb info for SYMBOL. With respect to the state diagram
+;;; presented at the definition of SYMBOL-PLIST, if the object in SYMBOL's
+;;; info slot is LISTP, it is in state 1 or 3. Either way, take the CDR.
+;;; Otherwise, it is in state 2 so return the value as-is.
+;;; In terms of this function being named "-vector", implying always a vector,
+;;; it is understood that NIL is a proxy for +NIL-PACKED-INFOS+, a vector.
+;;;
+;;; Declaim SYMBOL-INFO-VECTOR inline unless a vop translates it.
+;;; (Inlining occurs first, which would cause the vop not to be used.)
+;;; Also note that we have to guard the appearance of VOP-TRANSLATES here
+;;; so that it does not get tested when building the cross-compiler.
+;;; This was the best way I could see to work around a spurious warning
+;;; about a wrongly ordered VM definition in make-host-1.
+;;; The #!+/- reader can't see that a VOP-TRANSLATES term is not for the
+;;; host compiler unless the whole thing is one expression.
+#!-(or (host-feature sb-xc-host)
+       (vop-translates sb!kernel:symbol-info-vector))
+(declaim (inline symbol-info-vector))
 #-sb-xc-host
-(progn
- #!-symbol-info-vops (declaim (inline symbol-info-vector))
- (defun symbol-info-vector (symbol)
+(defun symbol-info-vector (symbol)
   (let ((info-holder (symbol-info symbol)))
     (truly-the (or null simple-vector)
-               (if (listp info-holder) (cdr info-holder) info-holder)))))
+               (if (listp info-holder) (cdr info-holder) info-holder))))
 
 ;;; SYMBOL-INFO is a primitive object accessor defined in 'objdef.lisp'
 ;;; But in the host Lisp, there is no such thing as a symbol-info slot.
 ;;; Instead, symbol-info is kept in the host symbol's plist.
+;;; This must be a SETFable place.
 #+sb-xc-host
 (defmacro symbol-info-vector (symbol) `(get ,symbol :sb-xc-globaldb-info))
 
@@ -254,7 +262,8 @@
 ;;;; functions and VOPs. To save space and allow for quick set
 ;;;; operations, we represent the attributes as bits in a fixnum.
 
-(deftype attributes () 'fixnum)
+(in-package "SB!C")
+(def!type attributes () 'fixnum)
 
 ;;; Given a list of attribute names and an alist that translates them
 ;;; to masks, return the OR of the masks.
@@ -286,21 +295,18 @@
         (test-name (symbolicate name "-ATTRIBUTEP")))
     `(progn
        (defmacro ,constructor (&rest attribute-names)
-         #!+sb-doc
          "Automagically generated boolean attribute creation function.
   See !DEF-BOOLEAN-ATTRIBUTE."
          (encode-attribute-mask attribute-names ,vector))
        (defun ,(symbolicate "DECODE-" name "-ATTRIBUTES") (attributes)
          (decode-attribute-mask attributes ,vector))
        (defmacro ,test-name (attributes &rest attribute-names)
-         #!+sb-doc
          "Automagically generated boolean attribute test function.
   See !DEF-BOOLEAN-ATTRIBUTE."
          `(logtest (the attributes ,attributes)
                    (,',constructor ,@attribute-names)))
        (define-setf-expander ,test-name (place &rest attributes
                                                &environment env)
-         #!+sb-doc
          "Automagically generated boolean attribute setter. See
  !DEF-BOOLEAN-ATTRIBUTE."
          (multiple-value-bind (temps values stores setter getter)
