@@ -39,7 +39,7 @@
 
 (eval-when (:compile-toplevel :load-toplevel :execute)
 (defvar *optimize-speed*
-  '(optimize (speed 3) (safety 0) (sb!ext:inhibit-warnings 3)))
+  '(optimize (speed 3) (safety 0) (sb!ext:inhibit-warnings 3) (debug 0)))
 ) ; EVAL-WHEN
 
 (defmacro dotimes-fixnum ((var count &optional (result nil)) &body body)
@@ -80,7 +80,7 @@
            ;; PCL's internal use of SET-FUNCALLABLE-INSTANCE-FUNCTION
            ;; doesn't obey this restriction.
            (type funcallable-instance fin))
-  (setf (funcallable-instance-fun fin) new-value))
+  (setf (%funcallable-instance-fun fin) new-value))
 
 ;;; FIXME: these macros should just go away.  It's not clear whether
 ;;; the inline functions defined by
@@ -89,10 +89,9 @@
 (declaim (inline fsc-instance-p))
 (defun fsc-instance-p (fin)
   (funcallable-instance-p fin))
-(defmacro fsc-instance-wrapper (fin)
-  `(%funcallable-instance-layout ,fin))
 (defmacro fsc-instance-slots (fin)
-  `(%funcallable-instance-info ,fin 1))
+  `(%funcallable-instance-info
+    ,fin ,(get-dsd-index standard-funcallable-instance clos-slots)))
 
 (declaim (inline clos-slots-ref (setf clos-slots-ref)))
 (declaim (ftype (function (simple-vector index) t) clos-slots-ref))
@@ -122,14 +121,13 @@
 ;; This is an absolutely terrible name for a function which both assigns
 ;; the name slot of a function, and _sometimes_ binds a name to a function.
 (defun set-fun-name (fun new-name)
-  #!+sb-doc
   "Set the name of a compiled function object. Return the function."
   (when (valid-function-name-p fun)
     (setq fun (fdefinition fun)))
   (typecase fun
-    (%method-function (setf (%method-function-name fun) new-name))
+    (%method-function fun)
     ;; a closure potentially becomes a different closure
-    (closure (setq fun (sb!impl::set-closure-name fun new-name)))
+    (closure (setq fun (set-closure-name fun t new-name)))
     (t (setf (%fun-name fun) new-name)))
   ;; Fixup name-to-function mappings in cases where the function
   ;; hasn't been defined by DEFUN.  (FIXME: is this right?  This logic
@@ -180,8 +178,8 @@
 ;;; weakening of STD-INSTANCE-P.
 ;;; FIXME: what does the preceding comment mean? You can't use instance-slots
 ;;; on a structure. (Consider especially a structure of 0 slots.)
-(defmacro std-instance-slots (x) `(%instance-ref ,x 1))
-(defmacro std-instance-wrapper (x) `(%instance-layout ,x))
+(defmacro std-instance-slots (x)
+  `(%instance-ref ,x ,(get-dsd-index standard-instance slots)))
 
 ;;; FIXME: These functions are called every place we do a
 ;;; CALL-NEXT-METHOD, and probably other places too. It's likely worth
