@@ -63,14 +63,8 @@ struct scan_state {
     struct scratchpad scratchpad;
 };
 
-static int gen_of(lispobj obj) {
-#ifdef LISP_FEATURE_IMMOBILE_SPACE
-    if (immobile_space_p(obj))
-        return immobile_obj_gen_bits(native_pointer(obj));
-#endif
-    int page = find_page_index((void*)obj);
-    if (page >= 0) return page_table[page].gen;
-    return -1;
+static int traceroot_gen_of(lispobj obj) {
+    return gc_gen_of(obj, -1);
 }
 
 static const char* classify_obj(lispobj ptr)
@@ -81,16 +75,16 @@ static const char* classify_obj(lispobj ptr)
     switch(lowtag_of(ptr)) {
     case INSTANCE_POINTER_LOWTAG:
         name = instance_classoid_name(native_pointer(ptr));
-        if (widetag_of(*name) == SIMPLE_BASE_STRING_WIDETAG)
+        if (widetag_of(name) == SIMPLE_BASE_STRING_WIDETAG)
           return (char*)(name + 2);
     case LIST_POINTER_LOWTAG:
         return "cons";
     case FUN_POINTER_LOWTAG:
     case OTHER_POINTER_LOWTAG:
-        return widetag_names[widetag_of(*native_pointer(ptr))>>2];
+        return widetag_names[widetag_of(native_pointer(ptr))>>2];
     }
     static char buf[8];
-    sprintf(buf, "#x%x", widetag_of(*native_pointer(ptr)));
+    sprintf(buf, "#x%x", widetag_of(native_pointer(ptr)));
     return buf;
 }
 
@@ -100,7 +94,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
     // Resurrect the containing object's lowtag
     lispobj ptr = compute_lispobj(obj);
     int staticp = ptr <= STATIC_SPACE_END;
-    int gen = staticp ? -1 : gen_of(ptr);
+    int gen = staticp ? -1 : traceroot_gen_of(ptr);
     if (heap_trace_verbose>2)
       // Show the containing object, its type and generation, and pointee
         fprintf(stderr,
@@ -121,7 +115,7 @@ static void add_to_layer(lispobj* obj, int wordindex,
 /// otherwise return obj directly.
 static lispobj canonical_obj(lispobj obj)
 {
-    if (functionp(obj) && widetag_of(*native_pointer(obj)) == SIMPLE_FUN_WIDETAG)
+    if (functionp(obj) && widetag_of((lispobj*)(obj-FUN_POINTER_LOWTAG)) == SIMPLE_FUN_WIDETAG)
         return make_lispobj(fun_code_header(obj-FUN_POINTER_LOWTAG),
                             OTHER_POINTER_LOWTAG);
     return obj;
@@ -142,7 +136,7 @@ static int find_ref(lispobj* source, lispobj target)
         check_ptr(1, source[1]);
         return -1;
     }
-    int widetag = widetag_of(header);
+    int widetag = widetag_of(source);
     scan_limit = sizetab[widetag](source);
     switch (widetag) {
     case INSTANCE_WIDETAG:
@@ -250,7 +244,7 @@ deduce_thread(void (*context_scanner)(), uword_t pointer, char** pc)
             }
         }
         if (!esp || esp == (void*) -1)
-            lose("deduce_thread: no SP known for thread %x (OS %x)", th, th->os_thread);
+            UNKNOWN_STACK_POINTER_ERROR("deduce_thread", th);
         void** where;
         for (where = ((void **)th->control_stack_end)-1; where >= esp;  where--)
             if ((uword_t)*where == pointer) {
@@ -404,9 +398,7 @@ static struct simple_fun* simple_fun_from_pc(char* pc)
 {
     struct code* code = (struct code*)component_ptr_from_pc((lispobj*)pc);
     if (!code) return 0;
-    struct simple_fun* prev_fun = (struct simple_fun*)
-        ((char*)code + (code_header_words(code->header)<<WORD_SHIFT)
-         + FIRST_SIMPLE_FUN_OFFSET(code));
+    struct simple_fun* prev_fun = 0;
     for_each_simple_fun(i, fun, code, 1, {
         if (pc < (char*)fun) break;
         prev_fun = fun;
@@ -419,7 +411,7 @@ static void maybe_show_object_name(lispobj obj, FILE* stream)
     extern void safely_show_lstring(lispobj* string, int quotes, FILE *s);
     lispobj package, package_name;
     if (lowtag_of(obj)==OTHER_POINTER_LOWTAG)
-        switch(widetag_of(*native_pointer(obj))) {
+        switch(widetag_of(native_pointer(obj))) {
         case SYMBOL_WIDETAG:
             putc(',', stream);
             if ((package = SYMBOL(obj)->package) == NIL) {
@@ -441,17 +433,18 @@ static boolean root_p(lispobj ptr, int criterion)
     // i.e. criterion 0 implies that that largest number of objects
     // are considered roots.
     return criterion < 2
-        && (gen_of(ptr) > (criterion ? HIGHEST_NORMAL_GENERATION
+        && (traceroot_gen_of(ptr) > (criterion ? HIGHEST_NORMAL_GENERATION
                            : gencgc_oldest_gen_to_gc));
 }
 
 /// Find any shortest path to 'object' starting at a tenured object or a thread stack.
-static void trace1(lispobj object,
-                   struct hopscotch_table* targets,
-                   struct hopscotch_table* visited,
-                   struct hopscotch_table* inverted_heap,
-                   struct scratchpad* scratchpad,
-                   int n_pins, lispobj* pins, void (*context_scanner)(),
+/// Return 1 if a path was found, 0 if not.
+static int trace1(lispobj object,
+                  struct hopscotch_table* targets,
+                  struct hopscotch_table* visited,
+                  struct hopscotch_table* inverted_heap,
+                  struct scratchpad* scratchpad,
+                  int n_pins, lispobj* pins, void (*context_scanner)(),
                    int criterion)
 {
     struct node* anchor = 0;
@@ -459,7 +452,7 @@ static void trace1(lispobj object,
     enum ref_kind root_kind;
     struct thread* root_thread;
     char* thread_pc = 0;
-    lispobj tls_index;
+    lispobj tls_index = 0;
     lispobj target;
     int i;
 
@@ -522,11 +515,16 @@ static void trace1(lispobj object,
         }
         if (!top_layer->count) {
             fprintf(stderr, "Failure tracing from %p. Current targets:\n", (void*)object);
-            for_each_hopscotch_key(i, target, (*targets))
-                fprintf(stderr, "%p ", (void*)target);
+            for_each_hopscotch_key(i, target, (*targets)) {
+                fprintf(stderr, "%p", (void*)target);
+                fprintf(stderr, "(g%d,", traceroot_gen_of(target));
+                fputs(classify_obj(target), stderr);
+                maybe_show_object_name(target, stderr);
+                fprintf(stderr,") ");
+            }
             putc('\n', stderr);
             free_graph(top_layer);
-            return;
+            return 0;
         }
         if (heap_trace_verbose>1)
             printf("Found %d object(s)\n", top_layer->count);
@@ -586,7 +584,7 @@ static void trace1(lispobj object,
             if (fun) {
                 fprintf(file, "fun=%p", (void*)make_lispobj(fun, FUN_POINTER_LOWTAG));
                 if (is_lisp_pointer(fun->name) &&
-                    widetag_of(*native_pointer(fun->name)) == SYMBOL_WIDETAG) {
+                    widetag_of(native_pointer(fun->name)) == SYMBOL_WIDETAG) {
                     fprintf(file, "=");
                     show_lstring(VECTOR(SYMBOL(fun->name)->name), 0, file);
                 }
@@ -606,7 +604,7 @@ static void trace1(lispobj object,
         if (ptr <= STATIC_SPACE_END)
             fprintf(file, "(static,");
         else
-            fprintf(file, "(g%d,", gen_of(ptr));
+            fprintf(file, "(g%d,", traceroot_gen_of(ptr));
         fputs(classify_obj(ptr), file);
         maybe_show_object_name(ptr, file);
         fprintf(file, ")#x%"OBJ_FMTX"[%d]->", ptr, next.wordindex);
@@ -619,13 +617,13 @@ static void trace1(lispobj object,
             break;
 #if FUN_SELF_FIXNUM_TAGGED
         case 1:
-            if (functionp(ptr) && widetag_of(*native_pointer(ptr)) == CLOSURE_WIDETAG)
+            if (functionp(ptr) && widetag_of(native_pointer(ptr)) == CLOSURE_WIDETAG)
                 target -= FUN_RAW_ADDR_OFFSET;
             break;
 #endif
         case 3:
             if (lowtag_of(ptr) == OTHER_POINTER_LOWTAG &&
-                widetag_of(FDEFN(ptr)->header) == FDEFN_WIDETAG)
+                widetag_of(&FDEFN(ptr)->header) == FDEFN_WIDETAG)
                 target = fdefn_callee_lispobj((struct fdefn*)native_pointer(ptr));
             break;
         }
@@ -643,6 +641,7 @@ static void trace1(lispobj object,
     }
     fprintf(file, "#x%"OBJ_FMTX".\n", target);
     fflush(file);
+    return 1;
 }
 
 static void record_ptr(lispobj* source, lispobj target,
@@ -691,7 +690,7 @@ static uword_t build_refs(lispobj* where, lispobj* end,
             check_ptr(where[1]);
             continue;
         }
-        int widetag = widetag_of(header);
+        int widetag = widetag_of(where);
         nwords = scan_limit = sizetab[widetag](where);
         switch (widetag) {
         case INSTANCE_WIDETAG:
@@ -744,6 +743,8 @@ static uword_t build_refs(lispobj* where, lispobj* end,
                     continue;
                 }
             }
+            if (is_vector_subtype(*where, VectorWeak))
+                continue;
             break;
         default:
             if (!(other_immediate_lowtag_p(widetag) && lowtag_for_widetag[widetag>>2]))
@@ -834,9 +835,9 @@ static void compute_heap_inverse(struct hopscotch_table* inverted_heap,
 /* Find any shortest path from a thread or tenured object
  * to each of the specified objects.
  */
-static void trace_paths(void (*context_scanner)(),
-                        lispobj weak_pointers, int n_pins, lispobj* pins,
-                        int criterion)
+static int trace_paths(void (*context_scanner)(),
+                       lispobj weak_pointers, int n_pins, lispobj* pins,
+                       int criterion)
 {
     int i;
     struct hopscotch_table inverted_heap;
@@ -845,6 +846,7 @@ static void trace_paths(void (*context_scanner)(),
     struct hopscotch_table visited;  // *Without* lowtag
     // A hashset of objects in the current graph layer
     struct hopscotch_table targets;  // With lowtag
+    int n_found = 0; // how many objects had paths to them
 
     if (heap_trace_verbose) {
         fprintf(stderr, "%d pins:\n", n_pins);
@@ -864,21 +866,24 @@ static void trace_paths(void (*context_scanner)(),
                 fprintf(stderr, "Target=%p (%s)\n", (void*)value, classify_obj(value));
             hopscotch_reset(&visited);
             hopscotch_reset(&targets);
-            trace1(value, &targets, &visited,
-                   &inverted_heap, &scratchpad,
-                   n_pins, pins, context_scanner, criterion);
+            n_found += trace1(value, &targets, &visited,
+                              &inverted_heap, &scratchpad,
+                              n_pins, pins, context_scanner, criterion);
         }
     } while (weak_pointers != NIL);
     os_invalidate(scratchpad.base, scratchpad.end-scratchpad.base);
     hopscotch_destroy(&inverted_heap);
     hopscotch_destroy(&visited);
     hopscotch_destroy(&targets);
+    return n_found;
 }
 
-void gc_prove_liveness(void(*context_scanner)(),
-                       lispobj objects,
-                       int n_pins, uword_t* pins,
-                       int criterion)
+/// Return number of sought objects that had paths to them.
+/// Return -1 for invalid input.
+int gc_prove_liveness(void(*context_scanner)(),
+                      lispobj objects,
+                      int n_pins, uword_t* pins,
+                      int criterion)
 {
     int n_watched = 0, n_live = 0, n_bad = 0;
     lispobj list;
@@ -886,7 +891,7 @@ void gc_prove_liveness(void(*context_scanner)(),
         ++n_watched;
         lispobj car = CONS(list)->car;
         if ((lowtag_of(car) != OTHER_POINTER_LOWTAG ||
-             widetag_of(*native_pointer(car)) != WEAK_POINTER_WIDETAG))
+             widetag_of(native_pointer(car)) != WEAK_POINTER_WIDETAG))
             ++n_bad;
         else
             n_live += ((struct weak_pointer*)native_pointer(car))->value
@@ -894,12 +899,12 @@ void gc_prove_liveness(void(*context_scanner)(),
     }
     if (!listp(list) || n_bad) {
         fprintf(stderr, "; Bad value in liveness tracker\n");
-        return;
+        return -1;
     }
     fprintf(stderr, "; Liveness tracking: %d/%d live watched objects\n",
             n_live, n_watched);
     if (!n_live)
-        return;
+        return 0;
     // Put back lowtags on pinned objects, since wipe_nonpinned_words() removed
     // them. But first test whether lowtags were already repaired
     // in case prove_liveness() is called after gc_prove_liveness().
@@ -909,17 +914,17 @@ void gc_prove_liveness(void(*context_scanner)(),
             pins[i] = compute_lispobj((lispobj*)pins[i]);
         }
     }
-    trace_paths(context_scanner, objects, n_pins, (lispobj*)pins, criterion);
+    return trace_paths(context_scanner, objects, n_pins, (lispobj*)pins, criterion);
 }
 
 /* This should be called inside WITHOUT-GCING so that the set
  * of pins does not change out from underneath.
  */
-void prove_liveness(lispobj objects, int criterion)
+int prove_liveness(lispobj objects, int criterion)
 {
     extern struct hopscotch_table pinned_objects;
     extern int gc_n_stack_pins;
-    gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
+    return gc_prove_liveness(0, objects, gc_n_stack_pins, pinned_objects.keys, criterion);
 }
 
 // These are slot offsets in (DEFSTRUCT THREAD),
