@@ -35,7 +35,8 @@
                (dolist (instruction (subst n :ARGC c-prog)
                                     (sb-assem::segment-buffer segment))
                  (apply (sb-assem::op-encoder-name (car instruction))
-                        (cdr instruction) segment (cdr instruction)))))))
+                        (cdr instruction) segment
+                        (sb-assem::perform-operand-lowering (cdr instruction))))))))
     (dotimes (n-args 4)
       (let ((the-code (assemble-it n-args)))
         ;; in case we change the way the assembler output works ...
