@@ -171,12 +171,12 @@
                     (the (complex single-float) y)))
                :stream (make-broadcast-stream)))
 
-;;; Data in the high bits of a fun header caused CODE-N-UNBOXED-DATA-WORDS
+;;; Data in the high bits of a fun header caused CODE-N-UNBOXED-DATA-BYTES
 ;;; to return a ridiculously huge value.
 (with-test (:name (disassemble :unboxed-data))
-  (assert (< (sb-kernel:code-n-unboxed-data-words
+  (assert (< (sb-kernel:code-n-unboxed-data-bytes
               (sb-kernel:fun-code-header #'expt))
-             100))) ; The exact value is irrelevant.
+             150))) ; The exact value is irrelevant.
 
 #+x86-64
 ;; The labeler for LEA would choke on an illegal encoding
@@ -206,8 +206,7 @@
     (assert (string= string1 string2)))))
 
 (with-test (:name :disassemble-assembly-routine)
-  (sb-disassem:disassemble-code-component sb-fasl:*assembler-routines*
-     :stream (make-broadcast-stream)))
+  (disassemble sb-fasl:*assembler-routines* :stream (make-broadcast-stream)))
 
 ;;; This tests that the x86-64 disasembler does not crash
 ;;; on LEA with a rip-relative operand and no label.
@@ -281,7 +280,8 @@
                           (invoke-restart (find-restart 'sb-kernel::call-form c) 123))))
     (assert (= (eval '(cons 1)) 123))))
 
-(with-test (:name :restart-bogus-arg-to-values-list-error)
+(with-test (:name :restart-bogus-arg-to-values-list-error
+                  :broken-on (not (or :x86 :x86-64 :arm :arm64)))
   (let ((fun (checked-compile `(lambda (x) (values-list x)))))
     (assert (equal (handler-bind ((sb-kernel::values-list-argument-error
                                    #'continue))
